/**
 * Modal dialogs node module.
 * Exports a function to instantiate a new dialog.
 * @param {Object} config config for the dialog
 * @param {Object} parent DOM element to append the dialog to
 * @param {Object} _window window of the dialog
 * @param {Function} callbackSubmit function to call when the dialog is closed with returnValue
 * @param {Function} callbackCancel function to call when the dialog is closed by cancellation, optional
 * @param {Bool} show immediately show the dialog, defaults to true
 * @return {Object} the _ModalDialog object
 */

module.exports = function ModalDialog(config, parent, _window, callbackSubmit, callbackCancel, show = true) {

	var _dialogElement,
		_returnValue = null,
		_doc = _window.document;

	/**
	 * Constructor
	 */
	var _ModalDialog = {};

	/**
	 * make _returnValue accessible from outside.
	 */
	_ModalDialog.getReturnValue = function() {
		return _returnValue;
	}

	/**
	 * Create a submit input element config object styled as a button
	 * @param {String} value value of the submit button
	 * @return {Object} submit button configuration object 
	 */
	function _createSubmitButton(value) {
		return {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'modalSubmit',
				'id': 'SUBMIT',
				'class': 'buttonBlue buttonsAlign',
				'value': value,
			}
		}
	}

	/**
	 * Create a button configuration object
	 * @param {String} value innerHTML for the button
	 * @param {String} id id for the button
	 * @param {String} listenerFn listener function for the button
	 * @return {Object} button configuration object
	 */
	function _createButton(value, id, listenerFn) {
		return {
			'element': 'button',
			'attributes': {
				'name': 'modalButton',
				'id': id,
				'class': 'buttonGrey buttonsAlign',
			},
			'innerHTML': value,
			'listeners': {
				'click': listenerFn,
			}
		}
	}

	/**
	 * Create a html div element for descriptions
	 * @param {String} value innerHTML of the div
	 * @return {Object} div configuration object
	 */
	function _createDescription(value) {
		return {
			'element': 'div',
			'innerHTML': value,
			'attributes': {
				'id': 'description',
			}
		}
	}

	/**
	 * Shortcut for 'Cancel' buttons
	 */
	var _modalCancelButton = _createButton(gt.gettext('Cancel'), 'CANCEL', _modalCancelHandler);

	/**
	 * Configuration object for a password input element
	 */
	var _inputPassword = {
		'element': 'input',
		'attributes': {
			'type': 'password',
			'name': 'modPassword',
			'id': 'modPassword',
			'required': true,
			'autocomplete': 'new-password',
			'placeholder': gt.gettext('Please enter your password.'),
		}
	};

	/**
	 * Configuration object for a username input element
	 */
	var _inputUsername = {
		'element': 'input',
		'attributes': {
			'type': 'text',
			'name': 'modUsername',
			'id': 'modUsername',
			'required': true,
			'autocomplete': 'nope',
			'placeholder': gt.gettext('Please enter your username.'),
		}
	};

	/**
	 * Configuration object for passwordDialog
	 * Asks the user for a password only.
	 */
	var _passwordDialog = {
		'ID': 'passwordDialog',
		'title': gt.gettext('Password'),
		'content': {
			'description': null,
			'password': _inputPassword,
		},
		'buttons': [
			_createSubmitButton(gt.gettext('OK')),
			_modalCancelButton,
		],
		'submitFn': _modalPasswordSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
		'focus': 'modPassword',
	}

	/**
	 * Configuration object for profileConfirmDialog
	 * Warns the user that unsaved changes are present.
	 * Choices: Save, Abort, Cancel
	 */
	var _profileConfirmDialog = {
		'ID': 'profileConfirmDialog',
		'title': gt.gettext('There are unsaved changes'),
		'content': {
			description: _createDescription(gt.gettext('Do you wish to apply the changes?')),
		},
		'buttons': [
			_createSubmitButton(gt.gettext('Save')),
			_createButton(gt.gettext('Discard'), 'DISCARD', _profileSubmitHandler),
			_modalCancelButton,
		],
		'submitFn': _profileSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
	}

	/**
	 * Configuration object for profileIncompleteDialog
	 * Warns the user that unsaved changes are present and also that the profile is incomplete.
	 * Choices: Cancel, Abort
	 */
	var _profileIncompleteDialog = {
		'ID': 'profileIncompleteDialog',
		'title': gt.gettext('There are unsaved changes'),
		'content': {
			description: _createDescription(gt.gettext('The profile settings are incomplete.')+'<br>'+gt.gettext('Do you want to continue editing your profile or discard current data?')),
		},
		'buttons': [
			_createSubmitButton(gt.gettext('Continue')),
			_createButton(gt.gettext('Discard'), 'DISCARD', _profileSubmitHandler),
		],
		'submitFn': _profileSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
	}

	/**
	 * Configuration object for profileDeleteDialog
	 * Lets the user confirm profile deletion.
	 * Choices: Delete, Cancel
	 */
	var _profileDeleteDialog = {
		'ID': 'profileDeleteDialog',
		'title': gt.gettext('Delete Profile'),
		'content':  {
			description: _createDescription(gt.gettext('Are you sure you want to delete this profile?')),
		},
		'buttons': [
			_createSubmitButton(gt.gettext('Delete')),
			_modalCancelButton,
		],
		'submitFn': _profileSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
	}

	/**
	 * Configuration object for credentialsDialog
	 * Asks the user for password and username.
	 */
	var _credentialsDialog = {
		'ID': 'credentialsDialog',
		'title': gt.gettext('Credentials needed.'),
		'content': {
			'description': null,
			'username': _inputUsername,
			'password': _inputPassword,
		},
		'buttons': [
			_createSubmitButton(gt.gettext('OK')),
			_modalCancelButton,
		],
		'submitFn': _modalCredentialsSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
		'focus': 'modUsername',
	}

	/**
	 * Configuration object for downloadDialog
	 * Asks the user what to do with a download.
	 * Choices: Save, Open, Cancel
	 */
	var _downloadDialog = {
		'ID': 'downloadDialog',
		'title': gt.gettext('File download'),
		'content': {
			description: _createDescription(gt.gettext('Do you want to open or save this file?')),
			details: {
				'element': 'div',
				'innerHTML': config.details,
				'attributes': {
					'id': 'details',
				}
			},
		},
		'buttons': [
			_createSubmitButton(gt.gettext('Open')),
			_createButton(gt.gettext('Save'), 'SAVE', _downloadSubmitHandler),
			_createButton(gt.gettext('Save as\u2026'), 'SAVEAS', _downloadSubmitHandler),
			_modalCancelButton,
		],
		'submitFn': _downloadSubmitHandler,
		'onCloseHandler': _modalOnCloseHandler,
		'focus': 'SUBMIT'
	}

	/**
	 * Eventhandler function for passwordDialog's submit button
	 * Closes a dialog with #password's value as return value.
	 * @param {Event} event event calling the function
	 */
	function _modalPasswordSubmitHandler(event) {
		event.preventDefault();
		_dialogElement.close(_doc.getElementById('modPassword').value);
	}

	/**
	 * Eventhandler function for profileConfirmDialog and profileDeleteDialog
	 * Also used for other buttons, than submit.
	 * Closes a dialog with SUBMIT as return value if the submit button was clicked.
	 * Otherwise the dialog is closed with the id of the button that has been clicked.
	 * @param {Event} event event calling the function
	 */
	function _profileSubmitHandler(event) {
		event.preventDefault();
		if (event.type === 'submit') {
			_dialogElement.close('SUBMIT');
		} else {
			_dialogElement.close(event.srcElement.id);
		}
	}

	/**
	 * General eventhandler function for cancel buttons
	 * Closes a dialog without return value.
	 * @param {Event} event event calling the function
	 */
	function _modalCancelHandler(event) {
		event.preventDefault();
		_dialogElement.close();
	}

	/**
	 * Eventlistener function for the 'close' event of a dialog
	 * Checks and sets the _returnValue of a _ModalDialog. Then calls the callback function.
	 * @param {Event} event event calling the function
	 */
	function _modalOnCloseHandler(event) {
		event.preventDefault();
		if (_dialogElement.returnValue) {
			_returnValue = _dialogElement.returnValue;
			callbackSubmit();
		} else if (callbackCancel){
			callbackCancel();
		}
		_dialogElement.remove();
	}

	/**
	 * Eventhandler function for credential dialog submit buttons
	 * Closes a dialog with #modPassword's and #modUsername's value as return value.
	 * @param {Event} event event calling the function
	 */
	function _modalCredentialsSubmitHandler(event) {
		event.preventDefault();
		_dialogElement.close(`{
			"password": "${_doc.getElementById('modPassword').value}",
			"username": "${_doc.getElementById('modUsername').value}"
		}`);

	}

	/**
	 * Eventhandler function for downloadDialog
	 * Closes a dialog with the action (save or open) and if this should be applied to settings. 
	 * @param {Event} event event calling the function
	 */
	function _downloadSubmitHandler(event) {
		event.preventDefault();
		if (event.type === 'submit') {
			_dialogElement.close(`{
				"action": "open"
			}`);
		} else if (event.type === 'click') {
			_dialogElement.close(`{
				"action": "${event.srcElement.id}"
			}`);
		} else {
			_dialogElement.close(`{
				"action": 'cancel'
			}`);
		}
	}

	/**
	 * Generic HTML Element creator function
	 * Creates and returns a HTML element form a config object.
	 * @param {Object} config config for the element
	 * @return {Object} the created HTML element
	 */
	function _createHTMLElement(config) {
		var el = document.createElement(config.element);
		var attributes = config.attributes;
		for (var attribute in attributes) {
			el.setAttribute(attribute, attributes[attribute]);
		}
		if (config.hasOwnProperty('label')){
			var label = document.createElement("label");
			label.innerHTML = config['label'];
			label.htmlFor = el.name;
			el.appendChild(label);
		}
		if (config.hasOwnProperty('innerHTML')){
			el.innerHTML = config['innerHTML'];
		}
		if (config.hasOwnProperty('checked')){
			el.checked = config['checked'];
		}
		if (config.hasOwnProperty('disabled')){
			el.disabled = config['disabled'];
		}
		if (config.hasOwnProperty('options')){
			for (var option in currentItem.options) {
				var opt = document.createElement("option");
				opt.text = config.options[option];
				opt.value = option;
				el.options.add(opt);
			}
		}
		if (config.hasOwnProperty('listeners')){
			var listeners = config['listeners'];
			for (var listener in listeners) {
				el.addEventListener(listener, listeners[listener]);
			}
		}
		return el;
	}

	/**
	 * Create and show modal dialogs from a config object.
	 * @param {Object} config configuration for the modal dialog
	 * @return {Object} the dialog HTMLElement
	 */
	function _modalDialog(config) {
		var d = document.createElement('dialog');
		d.id = config.ID;

		var titleBar = document.createElement('div');
		titleBar.className = 'titleBarModal';

		var title = document.createElement('div');
		title.className = 'titleText';
		title.innerHTML = config.title;

		var closeBtn = document.createElement('button');
		closeBtn.id = 'btClose';
		closeBtn.addEventListener('click', function(event) {
			d.close();
		});

		titleBar.appendChild(title);
		titleBar.appendChild(closeBtn);
		d.appendChild(titleBar);

		var content = document.createElement('form');
		content.id = 'modalContent';
		for (var item in config.content) {
			if (config.content[item] != null) {
				content.appendChild(_createHTMLElement(config.content[item]))
			}
		}

		var divButtons = document.createElement('div');
		divButtons.className = 'modalButtons';

		/* If we have two buttons, lower the distance between them */
		if (config.buttons.length === 2) {
			config.buttons[0].attributes.style = 'margin-right: 5px';
			config.buttons[1].attributes.style = 'margin-left: 5px';
		}
		for (let button of config.buttons) {
			divButtons.appendChild(_createHTMLElement(button))
		}
		content.appendChild(divButtons);
		content.addEventListener('submit', config.submitFn);
		d.appendChild(content);
		d.addEventListener('close', config.onCloseHandler);

		parent.appendChild(d);
		if (show) {
			d.showModal();
		}
		if (config.focus) {
			_doc.getElementById(config.focus).focus();
		}
		return d;
	}

	switch(config.type) {
		case 'passwordDialog':
			_dialogElement = _modalDialog(_passwordDialog);
			break;
		case 'passwordInvalidDialog':
			_passwordDialog.title = gt.gettext('Invalid password')
			_dialogElement = _modalDialog(_passwordDialog);
			break;
		case 'passwordForValidationDialog':
			_passwordDialog.content.description =
				_createDescription(gt.gettext('Your password is needed for profile validation.'))
			_dialogElement = _modalDialog(_passwordDialog);
			break;
		case 'profileConfirmDialog':
			_dialogElement = _modalDialog(_profileConfirmDialog);
			break;
		case 'profileIncompleteDialog':
			_dialogElement = _modalDialog(_profileIncompleteDialog);
			break;
		case 'profileDeleteDialog':
			_dialogElement = _modalDialog(_profileDeleteDialog);
			break;
		case 'credentialsDialog':
			_dialogElement = _modalDialog(_credentialsDialog);
			break;
		case 'downloadDialog':
			_dialogElement = _modalDialog(_downloadDialog);
			break;
		default:
			console.log("Unknown dialog type.");
			return null;
			break;
	}

	return _ModalDialog;
}
