/**
 * javascript for settings
 */

/* Global variables */
var left = document.getElementById("left");
var right = document.getElementById("right");

var isDirty = false;
var winPref = nw.Window.get();
var treeButtons;
var ModalDialog = global.DA.requireCustomModule('modalDialogs');
var downloads = global.DA.requireCustomModule('downloads');

/* Eventlistener for changes in the forms */
right.addEventListener('change', function(e) {
	isDirty = true;
});

/* show the menu tree on the left */
createTree();

/**
 * Add all buttons to the menu tree.
 */
function createTree() {
	global.DA.removeAllChildren(left);
	treeButtons = {};

	addTreeButton('general', gt.gettext('General'), 'buttonTree', createGeneralForm);
	addTreeButton('profiles', gt.gettext('Profiles'), 'buttonTree', createGeneralProfileSettingsForm);

	var profiles = global.settings.profiles;
	for (var profile in profiles) {
		addTreeButton(profiles[profile].id, profiles[profile].name, 'buttonTreeSub', createProfileForm);
	}
	addTreeButton('downloads', gt.gettext('Downloads'), 'buttonTree', createDownloadSettingsForm);
	addTreeButton('language', gt.gettext('Language'), 'buttonTree', createLanguageSettingsForm);
}

/**
 * Add a button to the div left, to build the menu tree
 * @param {String} id id of the button
 * @param {String} name name of the button
 * @param {String} type type of the button
 * @param {Function} listener listener function when the button is clicked
 * @param {Object} refNode if refnode is defined insert the child before it
 */
function addTreeButton(id, name, type, listener, refNode) {
	var btn = document.createElement("button");
	btn.className = type;
	btn.id = id;
	btn.innerText = name;
	btn.addEventListener('click', function(event) {
		event.preventDefault();
		if (this.classList.contains('active')) {
			return;
		}
		if (isDirty) {
			if (right.firstChild.checkValidity()) {
				var modal = new ModalDialog({
					'type': 'profileConfirmDialog'
				}, document.body, window, function() {
					isDirty = false;
					var ret = modal.getReturnValue();
					if (ret === 'DISCARD') {
						createTree();
					}
					if (ret === 'SUBMIT') {
						document.getElementById('submitbutton').click();
					}
					listener(null, btn.id);
					highlightTreeButton(btn.id);
				});
			} else {
				var modal = new ModalDialog({
					'type': 'profileIncompleteDialog'
				}, document.body, window, function() {
					var ret = modal.getReturnValue();
					if (ret === 'DISCARD') {
						isDirty = false;
						createTree();
						listener(null, btn.id);
						highlightTreeButton(btn.id);
					}
				});
			}
		} else {
			listener(event);
			highlightTreeButton(event.srcElement.id);
		}
	});
	if (refNode) {
		let newNode = left.insertBefore(document.createElement("br"), refNode);
		left.insertBefore(btn, newNode);
	} else {
		left.appendChild(btn);
		left.appendChild(document.createElement("br"));
	}
	treeButtons[id] = btn;
}

/**
 * Highlight a tree button by its id.
 * @param {String} buttonId id of the button to highlight
 */
function highlightTreeButton(buttonId) {
	for (var treeButton in treeButtons) {
		var curBtn = treeButtons[treeButton];
		if (curBtn.id == buttonId) {
			curBtn.className += ' active ';
		} else {
			curBtn.className = curBtn.className.replace(/ active /g, '');
		}
	}
}

/**
 * Eventlistener to directly update the profile name in the button tree
 * @param {Event} event event calling the function
 */
function updateTreeProfileName(event) {
	document.getElementById(event.srcElement.parentElement.id).innerHTML = event.srcElement.value;
}

/**
 * Create the form for the general profile settings
 * @param {Event} event event calling the function
 */
function createGeneralProfileSettingsForm(event) {
	var selectOptions = {};
	var profiles = global.settings.profiles;
	for (var profile in profiles) {
		selectOptions[profiles[profile].id] = profiles[profile].name;
	}
	var isAutoLoginEnabled = global.settings.general.autoLogin || false;

	var formConfig = {
		ID: 'defaultProfile',
		description: {
			'element': 'p',
			'attributes': {
				'class': 'settingDescription'
			},
			'innerHTML': gt.gettext('General profile settings')
		},
		profileSelector: {
			'element': 'select',
			'attributes': {
				'id': 'profileSelector',
				'name': 'profileSelector'
			},
			'label': gt.gettext('Default profile:'),
			'options': selectOptions
		},
		newProfile: {
			'element': 'button',
			'attributes': {
				'name': 'buttonNewProfile',
				'id': 'buttonNewProfile',
				'class': 'buttonGrey'
			},
			'innerHTML': gt.gettext('New Profile'),
			'listeners': {
				'click': newProfile,
			}
		},
		cbDefault: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbDefault',
				'id': 'cbDefault'
			},
			'checked': isAutoLoginEnabled,
			'label': gt.gettext('Automatically login with the default profile'),
		},
		submit: {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'submitbutton',
				'id': 'submitbutton',
				'class': 'buttonBlue',
				'value': gt.gettext('Save & Close')
			}
		},
		save: {
			'element': 'button',
			'attributes': {
				'name': 'buttonSave',
				'id': 'buttonSave',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Save'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					saveGeneralProfilesSettingsForm(null, false);
				},
			},
		},
	};

	global.DA.addForm(right, formConfig, saveGeneralProfilesSettingsForm);

	if (global.settings.general.defaultProfile) {
		document.getElementById('profileSelector').value = global.settings.general.defaultProfile;
	}
}

/**
 * Save the data from the profiles form
 * @param {Event} event event calling the function
 * @param {Boolean} close close the settings window after saving
 */
function saveGeneralProfilesSettingsForm(event, close = true) {
	if (event) {
		event.preventDefault();
	}
	var tmp_settings = {
		'general': {
			'defaultProfile': document.getElementById('profileSelector').value,
			'autoLogin': document.getElementById('cbDefault').checked
		}
	};
	global.DA.saveSettings(tmp_settings, true);
	winPref.onSettingsChange();
	isDirty = false;
	if (close) {
		winPref.close(true);
	}
}

/**
 * create a profile form for a profile id
 * Use event.srcElement.id or profileId to determine the profile to load.
 * @param {Event} event event calling the function
 * @param {String} profileId id of the profile to load
 */
function createProfileForm(event, profileId) {
	var profile = {};
	var profileName;
	if (profileId) {
		profile = getProfileById(profileId);
	} else if (event) {
		profile = getProfileById(event.srcElement.id);
		profileName = event.srcElement.innerHTML;
		profileId = event.srcElement.id;
	}

	var email = '';
	var username = '';
	var password = '';
	var url = '';
	var sso = false;
	var newProfile = true;
	var remember = true;

	if (Object.keys(profile).length != 0) {
		email = profile.email || '';
		if (profile.authType !== 'sso') {
			username = profile.requestOptions.form.username;
			password = DAEncryption.decrypt(profile.requestOptions.form.password);
		}
		url = profile.requestOptions.url.replace("index.php?logon", "");
		sso = (profile.authType === 'sso' ? true : false);
		remember = profile.rememberPassword;
		profileName = profile.name;
		newProfile = false;
	}

	if (newProfile) {
		isDirty = true;
	}

	var formConfig = {
		ID: profileId,
		profileName: {
			'element': 'input',
			'attributes': {
				'type': 'text',
				'name': 'profileName',
				'id': 'profileName',
				'value': profileName,
				'required': true,
				'maxlength': '60',
			},
			'label': gt.gettext('Profile name'),
			'listeners': {
				'input': updateTreeProfileName,
			}
		},
		email: {
			'element': 'input',
			'attributes': {
				'type': 'email',
				'name': 'email',
				'id': 'email',
				'value': email
			},
			'label': gt.gettext('Email address'),
			'listeners': {
				'blur': resolveMailAdress,
			}
		},
		username: {
			'element': 'input',
			'attributes': {
				'type': 'text',
				'name': 'username',
				'id': 'username',
				'required': true,
				'value': username,
				'autocomplete': 'nope',
			},
			'label': gt.gettext('Username'),
			'disabled': sso
		},
		password: {
			'element': 'input',
			'attributes': {
				'type': 'password',
				'name': 'password',
				'id': 'password',
				'required': true,
				'value': remember ? password : '',
				'autocomplete': 'new-password',
			},
			'label': gt.gettext('Password'),
			'disabled': sso || !remember
		},
		cbRememberPW: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbRememberPW',
				'id': 'cbRememberPW',
			},
			'label': gt.gettext('Remember password'),
			'checked': remember,
			'listeners': {
				'change': onCheckRemember,
			}
		},
		sso: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'sso',
				'id': 'sso',
			},
			'label': gt.gettext('Use system credentials'),
			'checked': sso,
			'listeners': {
				'change': checkSSO,
			}
		},
		url: {
			'element': 'input',
			'attributes': {
				'type': 'url',
				'name': 'url',
				'id': 'url',
				'value': url,
				'required': true
			},
			'label': gt.gettext('WebApp address')
		},
		cbDefault: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbDefaultProfile',
				'id': 'cbDefaultProfile',
			},
			'label': gt.gettext('Set as default profile')
		},
		error: {
			'element': 'div',
			'attributes': {
				'id': 'error'
			}
		},
		submit: {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'submitbutton',
				'id': 'submitbutton',
				'class': 'buttonBlue',
				'value': gt.gettext('Save & Close')
			}
		},
		save: {
			'element': 'button',
			'attributes': {
				'name': 'buttonSaveProfile',
				'id': 'buttonSaveProfile',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Save'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					saveProfileForm(null, profileId, null, false);
				},
			},
		},
		deleteProfile: {
			'element': 'button',
			'attributes': {
				'name': 'buttonDeleteProfile',
				'id': 'buttonDeleteProfile',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Delete profile'),
			'disabled': newProfile,
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					var notification = global.DA.requireCustomModule('notifications');
					notification.abortNotification();
					new ModalDialog({
						'type': 'profileDeleteDialog'
					}, document.body, window, function() {
						isDirty = false;
						deleteProfile(profileId);
						document.getElementById('profiles').click();
					});
				},
			},
		}
	};
	global.DA.addForm(right, formConfig, saveProfileForm);
}


/**
 * Create the form for the general settings
 * @param {Event} event event calling the function
 */
function createGeneralForm(event) {
	var hideOnMinimize = global.settings.general.hideOnMinimize || false;
	var closeOnLogout = global.settings.general.closeOnLogout || false;
	var formConfig = {
		ID: 'generalSettings',
		description: {
			'element': 'p',
			'attributes': {
				'class': 'settingDescription'
			},
			'innerHTML': gt.gettext('General settings')
		},
		cbMinimize: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbMinimize',
				'id': 'cbMinimize'
			},
			'checked': hideOnMinimize,
			'label': gt.gettext('Hide DeskApp when minimize window'),
		},
		cbCloseOnLogout: {
			'element': 'input',
			'attributes': {
				'type': 'checkbox',
				'name': 'cbCloseOnLogout',
				'id': 'cbCloseOnLogout'
			},
			'checked': closeOnLogout,
			'label': gt.gettext('Close DeskApp with the logout button'),
		},
		submit: {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'submitbutton',
				'id': 'submitbutton',
				'class': 'buttonBlue',
				'value': gt.gettext('Save & Close')
			}
		},
		save: {
			'element': 'button',
			'attributes': {
				'name': 'buttonSave',
				'id': 'buttonSave',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Save'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					saveGeneralForm(null, false);
				},
			},
		},
	};

	global.DA.addForm(right, formConfig, saveGeneralForm);
}

/**
 * Save the data from the general settings form
 * @param {Event} event event calling the function
 * @param {Boolean} close close the settings window after saving
 */
function saveGeneralForm(event, close = true) {
	if (event) {
		event.preventDefault();
	}
	var tmp_settings = {
		'general': {
			'hideOnMinimize': document.getElementById('cbMinimize').checked,
			'closeOnLogout': document.getElementById('cbCloseOnLogout').checked,
		}
	};
	global.DA.saveSettings(tmp_settings, true);
	winPref.onSettingsChange();
	isDirty = false;
	if (close) {
		winPref.close(true);
	}
}

/**
 * create a new profile
 * @param {Event} event event calling the function
 */
function newProfile(event) {
	if (event) {
		event.preventDefault();
	}
	if (isDirty) {
		return;
	}
	var newID = global.DA.newProfileID();
	var profileID = 'profile_' + newID;
	var profileName = gt.gettext('Profile') + ' ' + newID;
	addTreeButton(profileID, profileName, 'buttonTreeSub', createProfileForm, document.getElementById(
		'downloads'));
	document.getElementById(profileID).click();
}

/**
 * save a profile with a profile id
 * Use event.srcElement.id or profileId to determine the profile to save.
 * @param {Event} event event calling the function
 * @param {String} profileId id of the profile to save
 * @param {String} pw password for the profile
 * @param {Boolean} close close the settings window after saving
 */
function saveProfileForm(event, profileId, pw = '', close = true) {
	if (event) {
		event.preventDefault();
		profileId = event.srcElement.id;
	}

	var sso = document.getElementById("sso").checked;
	var setDefault = document.getElementById("cbDefaultProfile").checked;
	var error = document.getElementById("error");
	var remember = document.getElementById("cbRememberPW").checked;
	error.innerHTML = '';

	var password = pw || document.getElementById("password").value;
	if (!remember && password === '' && !sso) {
		var modalPW = new ModalDialog({
			'type': 'passwordForValidationDialog'
		}, document.body, window, function() {
			saveProfileForm(null, profileId, modalPW.getReturnValue(), close);
		});
		return;
	}

	var requestOptions = {
		url: document.getElementById("url").value.replace(/\/?$/, '/') + 'index.php?logon'
	};
	var tmpProfile = {
		'id': profileId,
		'name': document.getElementById("profileName").value,
		'email': document.getElementById("email").value,
		'passwordEncryption': DAEncryption.getVersion(),
		'rememberPassword': remember,
		'requestOptions': requestOptions,
	};
	if (!sso) {
		tmpProfile.requestOptions.form = {
			username: document.getElementById("username").value,
			password: DAEncryption.encrypt(password)
		};
	} else {
		tmpProfile.authType = 'sso';
	}
	if (setDefault) {
		global.DA.setAsDefault(profileId);
	}

	var winValidate = new WebAppWindow(tmpProfile,
		function(d) {
			console.log(d);
			global.DA.saveSettings({
				'profiles': {
					[d.profile.id]: d.profile
				}
			}, true);
			isDirty = false;
			document.getElementById("buttonDeleteProfile").disabled = false;
			winPref.onSettingsChange();
			if (close) {
				winPref.close(true);
			}
		},
		function(d) {
			console.log(d);
			var error = document.getElementById("error");
			error.innerHTML = d.message;
			error.style.visibility = '';
		});
	winValidate.validateAccountfromPreferences();
}

/**
 * Delete a profile by a given id
 * @param {String} profileId id of the profile to be deleted
 */
function deleteProfile(profileId) {
	console.log("Deleting profile: " + profileId);

	delete global.settings.profiles[profileId];

	/* If the deleted profile was default, set the first we can grab as default, or none */
	if (profileId === global.settings.general.defaultProfile) {
		var firstItem = Object.keys(global.settings.profiles)[0];
		if (firstItem) {
			global.settings.general.defaultProfile = global.settings.profiles[firstItem].id;
		} else {
			global.settings.general.defaultProfile = '';
		}
	}

	localStorage.setItem('settings', JSON.stringify(global.settings));
	winPref.onSettingsChange();

	createTree();
	isDirty = false;
	window.document.getElementById('profiles').click();
}

/**
 * Get the profile for a given id
 * @param {String} id id of the profile
 * @return {Object} the profile settings of id
 */
function getProfileById(id) {
	return global.settings['profiles'][id] || {};
}

/* Object to temporarily save credentials, for those who accidently click checkboxes */
var tmpCreds = {
	user: '',
	password: '',
	remember: undefined,
};

/**
 * Eventhandler function when the SSO checkbox is clicked
 */
function checkSSO() {
	var creds = {
		user: document.getElementById("username"),
		password: document.getElementById("password"),
	};

	for (var item in creds) {
		var curItem = creds[item];
		curItem.disabled = this.checked;
		curItem.required = !this.checked;
		if (this.checked) {
			tmpCreds[item] = curItem.value;
			curItem.value = '';
		} else {
			curItem.value = tmpCreds[item];
			tmpCreds[item] = '';
		}
	}

	var remember = document.getElementById("cbRememberPW");
	if (this.checked) {
		tmpCreds['remember'] = remember.checked;
		remember.checked = false;
	} else {
		remember.checked = tmpCreds['remember'];
		creds.password.disabled = !tmpCreds['remember'];
		tmpCreds['remember'] = undefined;
	}
	remember.disabled = this.checked;
}

var tmpPassword = '';
/**
 * Eventhandler function when the remember password checkbox is clicked
 */
function onCheckRemember() {
	var elPassword = document.getElementById("password");
	elPassword.disabled = !this.checked;
	if (this.checked) {
		elPassword.value = tmpPassword;
		tmpPassword = '';
	} else {
		tmpPassword = elPassword.value;
		elPassword.value = '';
	}
}

/**
 * create the download configuration form
 */
function createDownloadSettingsForm() {
	/* For backwards compatibility */
	if (typeof(global.settings.downloads.openOnCompleted) === 'boolean') {
		global.settings.downloads.downloadBehavior = global.settings.downloads.openOnCompleted ?
			'open' : 'ask';
		delete global.settings.downloads.openOnCompleted;
	}

	var formConfig = {
		ID: 'downloads',
		description: {
			'element': 'p',
			'attributes': {
				'class': 'settingDescription'
			},
			'innerHTML': gt.gettext('Download settings')
		},
               radioDlBehaviorOpen: {
                       'element': 'input',
                       'attributes': {
                               'type': 'radio',
                               'name': 'radioDlBehavior',
                               'id': 'radioDlBehaviorOpen',
                       },
                       'label': gt.gettext('Open downloads when completed'),
                       'checked': global.settings.downloads.downloadBehavior === 'open'
               },
               radioDlBehaviorAsk: {
                       'element': 'input',
                       'attributes': {
                               'type': 'radio',
                               'name': 'radioDlBehavior',
                               'id': 'radioDlBehaviorAsk',
                       },
                       'label': gt.gettext('Ask on every download'),
                       'checked': global.settings.downloads.downloadBehavior === 'ask'
               },
		downloadLocation: {
			'element': 'input',
			'attributes': {
				'type': 'text',
				'name': 'downloadLocation',
				'id': 'downloadLocation',
				'required': true,
				'value': downloads.getDefaultLocation(),
				'autocomplete': 'nope',
				'style': 'display: inline-block'
			},
			'label': gt.gettext('Default download directory'),
			'disabled': true
		},
		inputFolderSelector: {
			'element': 'input',
			'attributes': {
				'type': 'file',
				'nwdirectory': true,
				'nwworkingdir': downloads.getDefaultLocation(),
				'name': 'inputFolderSelector',
				'id': 'inputFolderSelector',
				'class': 'inputFolderSelector'
			},
			'listeners': {
				'change': function(event) {
					if (event.srcElement.value !== '') {
						document.getElementById('downloadLocation').value = event.srcElement.value;
					}
				},
			},
			'label': gt.gettext('Change\u2026')
		},
		temporaryDownloadLocation: {
			'element': 'input',
			'attributes': {
				'type': 'text',
				'name': 'temporaryDownloadLocation',
				'id': 'temporaryDownloadLocation',
				'required': true,
				'value': downloads.getTemporaryDownloadLocation(),
				'autocomplete': 'nope',
				'style': 'display: inline-block'
			},
			'label': gt.gettext('Directory for directly opening files'),
			'disabled': true
		},
		inputTemporaryFolderSelector: {
			'element': 'input',
			'attributes': {
				'type': 'file',
				'nwdirectory': true,
				'nwworkingdir': downloads.getTemporaryDownloadLocation(),
				'name': 'inputTemporaryFolderSelector',
				'id': 'inputTemporaryFolderSelector',
				'class': 'inputTemporaryFolderSelector'
			},
			'listeners': {
				'change': function(event) {
					if (event.srcElement.value !== '') {
						document.getElementById('temporaryDownloadLocation').value = event.srcElement.value;
					}
				},
			},
			'label': gt.gettext('Change\u2026')
		},
		buttonShowHistory: {
			'element': 'button',
			'attributes': {
				'name': 'buttonShowHistory',
				'id': 'buttonShowHistory',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Download history'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					downloads.openHistory();
				},
			},
		},
		submit: {
			'element': 'input',
			'attributes': {
				'type': 'submit',
				'name': 'submitbutton',
				'id': 'submitbutton',
				'class': 'buttonBlue',
				'value': gt.gettext('Save & Close')
			}
		},
		save: {
			'element': 'button',
			'attributes': {
				'name': 'buttonSave',
				'id': 'buttonSave',
				'class': 'buttonGrey',
			},
			'innerHTML': gt.gettext('Save'),
			'listeners': {
				'click': function(event) {
					event.preventDefault();
					saveDownloadSettingsForm(null, false);
				},
			},
		},
	};
	global.DA.addForm(right, formConfig, saveDownloadSettingsForm);
}


/**
 * save the downloads settings
 * @param {Event} event event calling the function
 * @param {Boolean} close close the settings window after saving
 */
function saveDownloadSettingsForm(event, close = true) {
	if (event) {
		event.preventDefault();
	}

	global.DA.saveSettings({
		'downloads': {
			'downloadBehavior': document.getElementById("radioDlBehaviorOpen").checked ? 'open' : 'ask',
			'temporaryDownloadLocation': document.getElementById("temporaryDownloadLocation").value,
		}
	}, true);
	downloads.setDefaultLocation(document.getElementById("downloadLocation").value);
	downloads.reloadDownloadSettings();
	isDirty = false;
	if (close) {
		winPref.close(true);
	}
}

/*
 * List of possible spellcheck Languages
 */
var spellcheckLanguages = {
	'af': gt.gettext('Afrikaans'),
	'bg': gt.gettext('Bulgarian'),
	'ca': gt.gettext('Catalan'),
	'cs': gt.gettext('Czech'),
	'da': gt.gettext('Danish'),
	'de': gt.gettext('German'),
	'el': gt.gettext('Greek'),
	'en-AU': gt.gettext('English (Australia)'),
	'en-CA': gt.gettext('English (Canada)'),
	'en-GB': gt.gettext('English (United Kingdom)'),
	'en-US': gt.gettext('English (United States)'),
	'es': gt.gettext('Spanish'),
	'et': gt.gettext('Estonian'),
	'fa': gt.gettext('Persian'),
	'fo': gt.gettext('Faroese'),
	'fr': gt.gettext('French'),
	'he': gt.gettext('Hebrew'),
	'hi': gt.gettext('Hindi'),
	'hr': gt.gettext('Croatian'),
	'hu': gt.gettext('Hungarian'),
	'id': gt.gettext('Indonesian'),
	'it': gt.gettext('Italian'),
	'ko': gt.gettext('Korean'),
	'lt': gt.gettext('Lithuanian'),
	'lv': gt.gettext('Latvian'),
	'nb': gt.gettext('Norwegian Bokm\u00e5l'),
	'nl': gt.gettext('Dutch'),
	'pl': gt.gettext('Polish'),
	'pt-BR': gt.gettext('Portuguese (Brazil)'),
	'pt-PT': gt.gettext('Portuguese (Portugal)'),
	'ro': gt.gettext('Romanian'),
	'ru': gt.gettext('Russian'),
	'sh': gt.gettext('Serbo-Croatian'),
	'sk': gt.gettext('Slovak'),
	'sl': gt.gettext('Slovenian'),
	'sq': gt.gettext('Albanian'),
	'sr': gt.gettext('Serbian'),
	'sv': gt.gettext('Swedish'),
	'ta': gt.gettext('Tamil'),
	'tg': gt.gettext('Tajik'),
	'tr': gt.gettext('Turkish'),
	'uk': gt.gettext('Ukrainian'),
	'vi': gt.gettext('Vietnamese')
};

/*
 * List of possible UI Languages
 */
var uiLanguages = {
	'en-GB': gt.gettext('English (United Kingdom)'),
	'en-US': gt.gettext('English (United States)'),
	'nl': gt.gettext('Dutch'),
	'de': gt.gettext('German'),
	'hu': gt.gettext('Hungarian'),
	'it': gt.gettext('Italian'),
	'nb': gt.gettext('Norwegian Bokm\u00e5l'),
	'pt-PT': gt.gettext('Portuguese (Portugal)')
}

/**
 * create the language configuration form
 */
function createLanguageSettingsForm() {
	chrome.settingsPrivate.getPref("spellcheck.dictionaries", function(details) {
		var selectedSpellcheckLanguages = details.value;
		var selectOptions = {
			'first': gt.gettext('Select Language\u2026')
		};
		var selectedSpellcheckLanguagesInnerHTML = '';
		for (let langId in spellcheckLanguages) {
			selectOptions[langId] = spellcheckLanguages[langId];
			if (selectedSpellcheckLanguages.includes(langId)) {
				selectedSpellcheckLanguagesInnerHTML += createSpellLangRow(langId);
			}
		}

		var formConfig = {
			ID: 'language',
			description: {
				'element': 'p',
				'attributes': {
					'class': 'settingDescription'
				},
				'innerHTML': gt.gettext('Language settings')
			},
			description2: {
				'element': 'p',
				'innerHTML': gt.gettext('Modify languages for spellchecking:'),
			},
			selectSpellLang: {
				'element': 'select',
				'attributes': {
					'id': 'selectSpellLang',
				},
				'options': selectOptions,
			},
			addSpellLang: {
				'element': 'button',
				'attributes': {
					'name': 'addSpellLang',
					'id': 'addSpellLang',
					'class': 'buttonGrey'
				},
				'innerHTML': gt.gettext('Add'),
				'listeners': {
					'click': function(e) {
						e.preventDefault();
						var selectEl = document.getElementById('selectSpellLang');
						if (selectEl.selectedIndex !== 0) {
							var langId = selectEl.value;
							document.getElementById('selectedSpellcheckLanguagesTable').innerHTML +=
								createSpellLangRow(langId);
							toggleSelectOptionByValue(langId);
							selectEl.selectedIndex = 0;
						}
					},
				}
			},
			selectedSpellcheckLanguagesTable: {
				'element': 'table',
				'attributes': {
					'id': 'selectedSpellcheckLanguagesTable'
				},
				'innerHTML': selectedSpellcheckLanguagesInnerHTML,
			},
			description3: {
				'element': 'p',
				'innerHTML': gt.gettext('Select UI language:'),
			},
			selectUiLang: {
				'element': 'select',
				'attributes': {
					'id': 'selectUiLang',
					'style': 'display: block'
				},
				'options': uiLanguages,
			},
			submit: {
				'element': 'input',
				'attributes': {
					'type': 'submit',
					'name': 'submitbutton',
					'id': 'submitbutton',
					'class': 'buttonBlue',
					'value': gt.gettext('Save & Close')
				}
			},
			save: {
				'element': 'button',
				'attributes': {
					'name': 'buttonSave',
					'id': 'buttonSave',
					'class': 'buttonGrey',
				},
				'innerHTML': gt.gettext('Save'),
				'listeners': {
					'click': function(event) {
						event.preventDefault();
						saveLanguageSettingsForm(null, false);
					},
				},
			},
		};
		global.DA.addForm(right, formConfig, saveLanguageSettingsForm);
		var firstSelectElement = document.getElementById('selectSpellLang').item(0);
		firstSelectElement.disabled = true;
		firstSelectElement.selected = true;
		firstSelectElement.hidden = true;
		for (let langId of selectedSpellcheckLanguages) {
			toggleSelectOptionByValue(langId);
		}

		if (global.settings.language && global.settings.language.defaultUiLang) {
			document.getElementById('selectUiLang').value = global.settings.language.defaultUiLang;
		}
	});
}

/**
 * Show or hide a select option, by its value.
 * @param {String} value of the option to toggle
 */
function toggleSelectOptionByValue(value) {
	var option = document.querySelector(`#selectSpellLang > option[value=${value}]`);
	option.hidden = !option.hidden;
}

/**
 * Create innerHTML for a spellcheck language row
 * @param {String} langId of the language to create a row for
 * @return {String} HTML for the row
 */
function createSpellLangRow(langId) {
	return `
		<tr id="${langId}" onMouseover="toggleRow(this)" onMouseout="toggleRow(this)">
			<td>
				<span>${spellcheckLanguages[langId]}</span>
				<button onclick="removeSpellLangRow('${langId}')" class="removeSpellLangBtn" hidden></button>
			</td>
		</tr>
	`
		.replace(/>\s+</g, "><").trim();
}

/**
 * Remove a spellcheck language row by its language id
 * @parama {string} langId the language id of the row that should be removed
 */
function removeSpellLangRow(langId) {
	document.getElementById(langId).remove();
	isDirty = true;
	toggleSelectOptionByValue(langId);
}

/**
 * Shows or hides the removeSpellLangBtn of a row
 * @param {Object} src the row for which the button should be toggled
 */
function toggleRow(src) {
	var btn = src.querySelector('.removeSpellLangBtn');
	btn.hidden = !btn.hidden;
	src.style.backgroundColor = btn.hidden ? '' : '#d5e8f7';
}

/**
 * save the language settings
 * @param {Event} event event calling the function
 * @param {Boolean} close close the settings window after saving
 */
function saveLanguageSettingsForm(event, close = true) {
	if (event) {
		event.preventDefault();
	}
	var languages = [];
	var table = document.getElementById('selectedSpellcheckLanguagesTable');
	for (let row of Array.from(table.getElementsByTagName('tr'))) {
		languages.push(row.id);
	}
	chrome.settingsPrivate.setPref("spellcheck.dictionaries", languages, "", function() {});
	chrome.settingsPrivate.setPref("intl.accept_languages", languages.join(), "", function() {});
	chrome.settingsPrivate.setPref("intl.app_locale", document.getElementById('selectUiLang').value, "", function() {});

	global.DA.saveSettings({
		'language' : {
			'defaultUiLang' : document.getElementById('selectUiLang').value
		}
	});

	var notification = global.DA.requireCustomModule('notifications');
	notification.showSaveNotification(document.body);
	isDirty = false;
	if (close) {
		winPref.close(true);
	}
}


/**
 * Event handler which resolve the host url from given email address
 * @param {Event} event event calling the function
 */
function resolveMailAdress(event) {
	event.preventDefault();
	if (!event.srcElement.value) {
		return;
	}
	var parts = event.srcElement.value.split('@');
	if (parts.length === 2) {
		const dns = require('dns');
		var domain = parts[1];
		dns.resolveTxt(domain, function(hostname, txts) {
			if (typeof txts !== 'undefined') {
				/* entries are [ [ ], [ ] ] */
				for (var entry of txts) {
					entry = entry[0];
					/* We are searching for 'kdiscover http://webapp.kopano.com/webapp/' */
					if (entry.indexOf('kdiscover') != -1 || entry.indexOf('zdiscover') != -1) {
						var entries = entry.split(' ');
						/* Second part, should be the domain.. */
						if (entries.length === 2) {
							var webapp_url = entries[1].trim();
							document.getElementById('url').value = webapp_url;
						}
					}
				}
			}
		});
	}
}
